from baelfire.task.task import Task

from baelfire.task.file import FileTask
from baelfire.task.process import SubprocessTask
from baelfire.task.template import TemplateTask

__all__ = [
    'Task',
    'FileTask',
    'TemplateTask',
    'SubprocessTask',
]
